
-- avoiding schema locks during partition switch

/*
SQL 2012 has an enhancement to Schema Stability locks that help here.  You will require a Sch-M lock 
to perform the partition switch, but you should acquire the lock after your 
last currently-running query completes, and incoming query requests will block until after the switch.

So if your longest-running queries are 30 seconds, your partition switch should 
take about 30 seconds and clients would see query runtime of up to 60 seconds.

In previous versions of SQL Server the pending Sch-M 
lock could be starved and the partition switch would time out.

Or you could try something like this to prevent blocking your queries, and delay the switch until a lull:
*/

set lock_timeout 5000
declare @try int = 0

while (1=1)
begin try
  set @try += 1
  alter table t switch to tt
  break
end try
begin catch

  if (@try > 10 or error_number() != 1222)
    throw 

end catch
