

use AdventureWorks2008R2
go

select * from Sales.SalesOrderDetail
go

exec sp_helpindex 'Sales.SalesOrderDetail'
go

drop index IdxLineTotal on Sales.SalesOrderDetail

-- clustered index, read all rows into Top N Sort
SELECT SalesOrderID, LineTotal
FROM Sales.SalesOrderDetail
ORDER BY LineTotal DESC
OFFSET 50 ROWS FETCH NEXT 10 ROWS ONLY;

-- Sort, segment-sequence, Filter
WITH CTE AS
(
SELECT SalesOrderID, LineTotal,
ROW_NUMBER() OVER(ORDER BY LineTotal DESC) AS rownumber
FROM Sales.SalesOrderDetail
)
SELECT *
FROM CTE
WHERE rownumber BETWEEN 51 AND 60
ORDER BY rownumber;

-- Later rows
SELECT SalesOrderID, LineTotal
FROM Sales.SalesOrderDetail
ORDER BY LineTotal DESC
OFFSET 100000 ROWS FETCH NEXT 10 ROWS ONLY;

-- Rownumber is a little better
WITH CTE AS
(
SELECT SalesOrderID, LineTotal,
ROW_NUMBER() OVER(ORDER BY LineTotal DESC) AS rownumber
FROM Sales.SalesOrderDetail
)
SELECT *
FROM CTE
WHERE rownumber BETWEEN 100001 AND 100060
ORDER BY rownumber;

-- create covering index
CREATE INDEX IdxLineTotal ON Sales.SalesOrderDetail(LineTotal)
go

-- OFFSET/FETCH is better
SELECT SalesOrderID, LineTotal
FROM Sales.SalesOrderDetail
ORDER BY LineTotal DESC
OFFSET 50 ROWS FETCH NEXT 10 ROWS ONLY;

-- Sort, segment-sequence, Filter
WITH CTE AS
(
SELECT SalesOrderID, LineTotal,
ROW_NUMBER() OVER(ORDER BY LineTotal DESC) AS rownumber
FROM Sales.SalesOrderDetail
)
SELECT *
FROM CTE
WHERE rownumber BETWEEN 51 AND 60
ORDER BY rownumber;


SELECT SalesOrderID, LineTotal
FROM Sales.SalesOrderDetail
ORDER BY LineTotal DESC
OFFSET 100000 ROWS FETCH NEXT 10 ROWS ONLY;

-- CTE - bad estimate
WITH CTE AS
(
SELECT SalesOrderID, LineTotal,
ROW_NUMBER() OVER(ORDER BY LineTotal DESC) AS rownumber
FROM Sales.SalesOrderDetail
)
SELECT *
FROM CTE
WHERE rownumber BETWEEN 100001 AND 100060
ORDER BY rownumber;

-- no covering index
SELECT SalesOrderID, LineTotal, rowguid
FROM Sales.SalesOrderDetail
ORDER BY LineTotal DESC
OFFSET 50 ROWS FETCH NEXT 10 ROWS ONLY;

WITH CTE AS
(
SELECT SalesOrderID, LineTotal, rowguid,
ROW_NUMBER() OVER(ORDER BY LineTotal DESC) AS rownumber
FROM Sales.SalesOrderDetail
)
SELECT *
FROM CTE
WHERE rownumber BETWEEN 51 AND 60
ORDER BY rownumber;

-- uses clustered index
SELECT SalesOrderID, LineTotal, rowguid
FROM Sales.SalesOrderDetail
ORDER BY LineTotal DESC
OFFSET 100050 ROWS FETCH NEXT 10 ROWS ONLY;

-- uses lookup, much worse
WITH CTE AS
(
SELECT SalesOrderID, LineTotal, rowguid,
ROW_NUMBER() OVER(ORDER BY LineTotal DESC) AS rownumber
FROM Sales.SalesOrderDetail
)
SELECT *
FROM CTE
WHERE rownumber BETWEEN 100051 AND 100060
ORDER BY rownumber;
